﻿/**
 * @file hv_cal_Physical.h
 * @brief PHY driver layer interface.
 * @verbatim
 * ==============================================================================
 *                        ##### How to use this driver #####
 * ==============================================================================
 * (+) Use HS_HAL_WDT_Start(...) to start the wdt immediately.
 * (+) Use HS_HAL_WDT_Stop(...) to stop the wdt immediately.
 * (+) Use HS_HAL_WDT_Feed(...) to feed the watchdog and it will restart the counter immediately. 
 * @endverbatim
 *
 * @author HiView SoC Software Team
 * @version 0.0.1
 * @date 2022-08-22
 */

#ifndef __HV_CAL_PHYSICAL_H__
#define __HV_CAL_PHYSICAL_H__

#include "Common/hv_comm_DataType.h"


Status Hv_Cal_Phy_HsrxInit(UCHAR8 ucPortIndex);

/* PHY的数字部分初始化 */
Status Hv_Cal_Phy_DigitalInit();

/* PHY的模拟部分初始化 */
Status Hv_Cal_Phy_AnalogInit();

Status Hv_Cal_Phy_SetRxMapping(UCHAR8 ucPortIndex, UINT32 uiValue);

Status Hv_Cal_Phy_DisableRxMapping(UCHAR8 ucPortIndex);

VOID Hv_Cal_System_Reset_En(VOID);

#endif

