/**
 * @file  hv_drv_TypecStdInc.h
 * @brief   PSF configuration header file
 *    This header file contains the configuration parameters of PSF stack that are 
 *    required to configure the Power delivery modules.
 * @details This file provides the following functions: \n
 *          (1) \n
 *          (2) \n
 *          (3) \n
 *
 * @author HiView SoC Software Team
 * @version 1.0.0
 * @date 2022-08-25
 * @copyright Copyright(c),2022-8, Hiview Software. All rights reserved.
 * @par History:
 * <table>
 * <tr><th>Author                         <th>Date         <th>Change Description
 * <tr><td>Hiview SoC Software Team       <td>2022-08-25   <td>init
 * </table>
*/

#ifndef _HV_DRV_TYPECSTDINC_H_
#define _HV_DRV_TYPECSTDINC_H_

#include "hv_vos_Comm.h"
#include "hv_comm_Event.h"
#include "HV_REGS_HT7315.h"

#include "TypecConfig.h"

#include "hv_drv_TypecGenericDefs.h"
#if (HV_TYPEC_CONFIG_TYPEC_MODE == TYPEC_MODE_DRP)
#include "hv_drv_TypecConfigDrp.h"
#elif (HV_TYPEC_CONFIG_TYPEC_MODE == TYPEC_MODE_SRC)
#include "hv_drv_TypecConfigSrc.h"
#elif (HV_TYPEC_CONFIG_TYPEC_MODE == TYPEC_MODE_SINK)
#include "hv_drv_TypecConfigSink.h"
#else
#error "Typec-c mode must be configured."
#endif
#include "hv_drv_TypecApiHook.h"
#include "hv_drv_TypecPowerControl.h"
#include "hv_drv_TypecDebugDefs.h"
#include "hv_drv_TypecTimer.h"
#include "hv_drv_TypecControl.h"
#include "hv_drv_TypecPolicyManager.h"
#include "hv_drv_TypecPolicyEngine.h"
#include "hv_drv_TypecInterrupts.h"
#include "hv_drv_TypecHardware.h"
#include "hv_drv_TypecProtocolLayer.h"
#if (HV_TRUE == INCLUDE_POWER_BALANCING)
#include "hv_drv_TypecPbMngr.h"
#endif
#if (HV_TRUE == INCLUDE_POWER_THROTTLING)
#include "pt_mngr.h"
#endif 
#include "hv_drv_TypecInternalGlobals.h"
#include "hv_drv_TypecPolicyEngineFwUp.h"
#include "hv_drv_TypecProjectVersion.h"
#include "hv_drv_TypecTimeoutDefs.h"

#endif