﻿/*
 * @file hv_mw_OsdInit.h
 * @brief Header file of osd init.
 *
 * @verbatim
 * ==============================================================================
 *                        ##### How to use #####
 * ==============================================================================
 * (+) Use Hv_Mw_OsdIsInited() to check osd if inited.
 * (+) Use Hv_Mw_OsdInit() to init osd module.
 * (+) Use Hv_Mw_OsdDeinit() to de-init osd module.
 * (+) Use Hv_Mw_OsdPostInit() to post-init osd module.
 *
 * @endverbatim
 * @author HiView SoC Software Team
 * @version 1.0.0
 * @date 2022-09-01
 */

#ifndef _HV_MW_OSD_INIT_H
#define _HV_MW_OSD_INIT_H

#include "hv_mw_OsdComm.h"

//#define OSD_DISABLE

 /**
  * @brief OSD 是否初始化完成
  * @return BOOL  返回值：是-HV_TRUE，失败-HV_FALSE
  */
BOOL Hv_Mw_OsdIsInited(VOID);

/**
 * @brief init OSD
 * @return Status  返回值：成功-HV_SUCCESS，失败-HV_FAILURE
 */
Status Hv_Mw_OsdInit(VOID);

/**
 * @brief deinit OSD
 */
VOID Hv_Mw_OsdDeinit(VOID);

/**
 * @brief 后初始化Osd，包括：开启中断，加载资源，创建scene/window/item数据
 * @return Status  返回值：成功-HV_SUCCESS，失败-HV_FAILURE
 */
Status Hv_Mw_OsdPostInit(VOID);

#endif
