/**
 * @file Pinshare.h
 * @brief Header file of Project.
 *
 * @verbatim
 * ==============================================================================
 *                        ##### How to use #####
 * ==============================================================================
 *
 *
 @ endverbatim
 *
 * @author HiView SoC Software Team
 * @version 1.0.0
 * @date 2023-05-11
 */
#ifndef __HV_MW_LED_H__
#define __HV_MW_LED_H__
#include "Common/hv_comm_DataType.h"

/**
 * @brief 设置待机后指示灯的运行模式，呼吸or关闭
 * @param[in] bMode 0:off 1:on 2:breath
 * @return none.
 */
VOID Hv_Mw_Led_SetLedPowerMode(UCHAR8 bMode);
/**
 * @brief 设置指示红灯的状态.
 * @param[in] ucMode 0-5.
 * @return none.
 */
VOID Hv_Mw_Led_SetRedLedMode(UCHAR8 ucMode);

/**
 * @brief 设置开机状态下指示红灯的工作状态，亮或灭.
 * @param[in] None.
 * @return none.
 */
VOID Hv_Mw_Led_SetRedLed(BOOL bEn);
/**
 * @brief 设置开机状态下指示蓝灯的工作状态，亮或灭.
 * @param[in] None.
 * @return none.
 */
VOID Hv_Mw_Led_SetBlueLed(BOOL bEn);
/**
 * @brief 设置开机状态下指示蓝灯的亮度.
 * @param[in] ucBrightness亮度.
 * @return none.
 */
VOID Hv_Mw_Led_SetBlueLedBright(UCHAR8 ucBrightness);

/**
 * @brief 设置不同状态下的灯效状态.
 * @param[in] eMode 开机，待机或者工厂模式下.
 * @return none.
 */
VOID Hv_Mw_Led_SetLedMode(LedMode eMode);
#endif
